import es
import playerlib
import math
import gamethread

import psyco
psyco.full()

#plugin information
info = es.AddonInfo()
info.name = "Popup EventScripts Python library"
info.version = "oy19"
info.author = "GODJonez + ES 2.0 Beta Test team"
info.url = "http://www.eventscripts.com/pages/Popup/"
info.description = "Provides menu and pop-up handling for Source games"

#global variables:
## gLanguage and gGamename are used by popup.construct and popup.easymenu
gLanguage = "en"
gGamename = "cstrike"
## gPopups holds all the popups by their names for backwards compatibility
gPopups = {}
## gUsers holds user-specific information such as queues, defined later

# Static data to be displayed in popup generated content:
langdata = {}
langdata['en'] = {'prev': "Back", 'next': "Next", 'quit': "Cancel", 'pending': "Pending"}                               #by [NATO]Hunter
langdata['fr'] = {'prev': "Retour", 'next': "Suivant", 'quit': "Annuler", 'pending': "En attente"}                       #by Nicolous
langdata['de'] = {'prev': "Zur\xc3\xbcck", 'next': "Weiter", 'quit': 'Schlie\xc3\x9fen', 'pending': "Wechselnd"}       #by [NATO]Hunter
langdata['fi'] = {'prev': 'Edellinen', 'next': 'Seuraava', 'quit': 'Sulje', 'pending': 'Odottaa'}                       #by GODJonez
teamdata = {}
teamdata['cstrike'] = {0: "UN", 1: "SPEC", 2: "T", 3: "CT"}
teamdata['dod'] = {0: "UN", 1: "SPEC", 2: "AL", 3: "AX"}
cbotdata = {0: "HUMAN", 1: "BOT"}
cdeaddata = {0: "", 1: "DEAD"}

# Get this library handle for registering events dynamically:
selfmodule = __import__('popuplib')



class Popup_base(object):
    '''
    Popup_base class is the base of all popup types,
    do not use this class directly from anywhere except for
    checking variable type
    '''
    def __init__(self, pPopupid):
        '''
        Initialize the popup by creating necessary attributes
        '''
        self.name = pPopupid            #popup name for backwards compatibility
        self.timeoutsend = 0            #timeout in seconds for being in queue
        self.timeoutview = 0            #timeout for being visible
        self.prepuser = ""              #prepuser block
        self.oldskooltov = False        #old (ES 1.2 - ES 1.3) behavior on timeout view
        self.editlang = gLanguage       #Language to be edited
    def i_userlang(self, userid):
        '''
        INTERNAL
        Returns a usable language code from cache,
        tries to conform to the language the user itself is using
        '''
        user = gUsers[int(userid)]
        if user:
            userlang = user.language
            if not self.cache.has_key(userlang):
                if self.cache.has_key(gLanguage):
                    return gLanguage
                else:
                    k = self.cache.keys()
                    if len(k) == 0:
                        return None
                    else:
                        return k[0]
            return userlang
        return None
    def i_endlang(self, userid):
        '''
        INTERNAL
        User wrapper for i_endlangg
        '''
        return self.i_endlangg(gUsers[int(userid)].language)
    def i_endlangg(self, userlang):
        '''
        INTERNAL
        Returns usable language code for constructable strings,
        tries to conform to the language passed in
        '''
        if not langdata.has_key(userlang):
            if langdata.has_key(gLanguage):
                return gLanguage
            else:
                k = langdata.keys()
                if len(k) == 0:
                    return None
                else:
                    return k[0]
        return userlang
    def delete(self):
        '''
        delete this popup, calls global delete function
        '''
        delete(self.name)
    def timeout(self, mode, time):
        '''
        set send/view timeout
        '''
        if (time >= 0):
            if (mode == "send"):
                self.timeoutsend = time
            elif (mode == "view"):
                self.timeoutview = time
            else:
                raise ValueError("Popuplib: Unsupported timeout mode: "+str(mode))
        else:
            raise ValueError("Popuplib: Invalid time value for timeout: "+str(time))
    def send(self, pUsers, prio=False):
        '''
        send this popup to users queue
        '''
        try:
            # try if pUsers is just a single userid
            userid = int(pUsers)
            pUsers = [userid]
        except TypeError:
            pass
        for userid in pUsers:
            #Priorization system:
            # if the user just used a chat or client command, he requested this popup to be sent
            # so send it immediately and put the older one to queue
            user = gUsers[int(userid)]
            if not user:
                continue
            es.dbgmsg(1,"Popuplib: send to "+str(userid)+" with prio="+str(prio))
            es.dbgmsg(1,"Popuplib: user.prioritized="+str(user.prioritized))
            if not user.prioritized:
                try:
                    testuserid = int(es.getEventInfo('userid'))
                    if userid == testuserid:
                        cevent = str(es.getEventInfo('text'))
                        if cevent != "":
                            prio = True
                        if str(es.getEventInfo('es_event')) == "es_client_command":
                            prio = True
                except (ValueError, TypeError):
                    pass
                try:
                    es.set('testuserid','0')
                    es.server.cmd('es_xgetcmduserid testuserid')
                    if userid == int(es.server_var['testuserid']):
                        prio = True
                except TypeError:
                    pass
            if prio:
                if user.active:
                    index = user.activeid - 1
                    user.queue[user.activeid] = user.active
                    user.queue[index] = self
                    user.active = None
                else:
                    prio = False
                user.prioritized = True
                gamethread.queue(gt_priofinish, user)
            if not prio:
                index = user.nextindex
                user.nextindex += 1
                user.queue[index] = self
            if not user.closemode:
                # if in closemode, soon there will be menuselect 10!
                user.handleQueue()
            if self.timeoutsend > 0:
                gamethread.delayed(self.timeoutsend, unsend, (index, int(userid)))
                
    def unsend(self, pUsers):
        '''
        remove this popup from users queue and display
        '''
        try:
            user = gUsers[int(pUsers)]
            if user: user.unsend(self, True)
        except TypeError:
            for userid in pUsers:
                user = gUsers[int(userid)]
                if user: user.unsend(self, True)
        
    def update(self, pUsers):
        '''
        redisplay this popup if its the visible one
        '''
        es.dbgmsg(1,"Popuplib: update("+str(pUsers)+str(type(pUsers))+")")
        try:
            user = gUsers[int(pUsers)]
            if user: user.updatePopup(self)
        except TypeError:
            for userid in pUsers:
                user = gUsers[int(userid)]
                if user: user.updatePopup(self)
    

class Popup_popup(Popup_base):
    '''
    Popup_popup is the normal popup class that inherits Popup_base
    by this it has automatically access to the methods and variables
    defined in Popup_base class
    '''
    def __init__(self, pPopupid):
        '''
        Initialize the popup by creating all necessary attributes
        with their default values
        '''
        # First call the Popup_base initialization:
        Popup_base.__init__(self, pPopupid)
        # Initialize variables
        self.lines = {}                 #contains the lines to be shown (inside languages)
        self.menuselect = ""            #menuselect block
        self.menuselectfb = ""          #menuselect fallback block
        self.p_select = {}              #dictionary of select blocks
        self.p_submenu = {}             #dictionary of submenu choices
        self.menuvar = {}               #menu variables
        self.menuval = {}               #menu values
        self.cachemode = "global"       #cachemode, allowed: "global", "static", "user"
        self.cache = {}                 #cached popup text
        self.cachere = {}               #boolean to show if any changes were made since last cache
        self.displaymode = "normal"     #mode, "normal" or "sticky"

        self.editlang = "default"
        self.lines["default"] = {}
        self.cache["default"] = "Uncached %s"%self.name
        self.cachere["default"] = False
        
        self.type = "popup"
    def checklang(self, lang=None):
        '''
        INTERNAL
        Checks if the current editlang exists and if not,
        creates structures for it
        '''
        if not lang:
            editlang = self.editlang
        else:
            editlang = str(lang)
        es.dbgmsg(1, "Popuplib: checklang %s"%editlang)
        if not self.lines.has_key(editlang):
            es.dbgmsg(1, "Popuplib:  NOT FOUND!")
            self.lines[editlang] = self.lines["default"].copy()
            self.cache[editlang] = str(self.cache["default"])
            self.cachere[editlang] = bool(self.cachere["default"])
    def information(self, listlevel):
        '''
        Output information about this popup to server console
        '''
        if listlevel >= 1:
            es.dbgmsg(0, " ")
        es.dbgmsg(0, self.name)
        if listlevel >= 1:
            es.dbgmsg(0, "  Type:         "+str(self.type))
            es.dbgmsg(0, "  Lines:        "+str(len(self.lines)))
            es.dbgmsg(0, "  Prepuser:     "+str(self.prepuser))
            es.dbgmsg(0, "  Menuselect:   "+str(self.menuselect))
            es.dbgmsg(0, "  Menuselectfb: "+str(self.menuselectfb))
        if listlevel >= 2:
            es.dbgmsg(0, "  Select:       "+str(len(self.p_select)))
            for item in self.p_select:
                es.dbgmsg(0,"    ["+str(item)+"]:  "+str(self.p_select[item]))
            es.dbgmsg(0, "  Submenu:      "+str(len(self.p_submenu)))
            for item in self.p_submenu:
                es.dbgmsg(0,"    ["+str(item)+"]:  "+str(self.p_submenu[item]))
        if listlevel >= 3:
            es.dbgmsg(0, "  Menuvalue:    "+str(len(self.menuvar)))
            for item in self.menuvar:
                es.dbgmsg(0,"    ["+str(item)+"]:  "+str(self.menuvar[item])+'="'+str(self.menuval[item])+'"')
        if listlevel >= 2:
            es.dbgmsg(0, "  Cachemode:    "+str(self.cachemode))
            es.dbgmsg(0, "  Cachere:      "+str(self.cachere))
            es.dbgmsg(0, "  Displaymode:  "+str(self.displaymode))
        if listlevel >= 3:
            es.dbgmsg(0, "  Current content:")
            for lang in self.lines:
                es.dbgmsg(0, "   Language: %s"%lang)
                for line in self.lines[lang]:
                    es.dbgmsg(0, str(self.lines[lang][line]))
        if listlevel >= 4:
            es.dbgmsg(0, "  Current globally cached content:")
            for lang in self.cache:
                es.dbgmsg(0, "   Language: %s"%lang)
                cc = self.cache[lang].split("\\n")
                for line in cc:
                    es.dbgmsg(0, line)
    def isValidLine(self, line, editlang):
        '''
        INTERNAL
        checks if the line in question is valid for editing
        '''
        if (line < 1): return False
        if (line > len(self.lines[editlang])): return False
        return True
    def isValidItem(self, line):
        '''
        INTERNAL
        checks if the item in question is valid menu choice
        '''
        if (line < 1): return False
        if (line > 10): return False
        return True
    def addline(self, text, lang=None):
        '''
        add a line of text to the end of the list
        '''
        if isinstance(text, dict):
            # Assume it is a dict-type object that has languages as keys
            for editlang in text:
                self.checklang(editlang)
                self.lines[editlang][len(self.lines[editlang])+1] = text[editlang]
        else:
            if not lang:
                editlang = self.editlang
            else:
                editlang = str(lang)
            self.checklang(editlang)
            self.lines[editlang][len(self.lines[editlang])+1]=text
            self.cachere[self.editlang]=True
    def delline(self, line, lang=None):
        '''
        delete a line of text and move subsequent lines
        '''
        if not lang:
            editlang = self.editlang
        else:
            editlang = str(lang)
        self.checklang(editlang)
        if (self.isValidLine(line, editlang)):
            lines = len(self.lines[editlang])
            for thisline in range(line,lines):
                self.lines[editlang][thisline] = self.lines[editlang][thisline+1]
            del self.lines[editlang][lines]
            self.cachere[editlang]=True
        else:
            raise IndexError("Popuplib: Cannot delete line #%d from %s"%(line,self.name))
    def insline(self, line, text, lang=None):
        '''
        insert a line and make room for it
        '''
        if not lang:
            editlang = self.editlang
        else:
            editlang = str(lang)
        self.checklang(editlang)
        lines = len(self.lines[editlang])
        if (line==lines+1 or self.isValidLine(line,editlang)):
            for thisline in range(lines+1,line,-1):
                self.lines[editlang][thisline] = self.lines[editlang][thisline-1]
            self.lines[editlang][line] = text
            self.cachere[editlang]=True
        else:
            raise IndexError("Popuplib: Cannot insert line #%d to %s"%(line,self.name))
    def modline(self, line, text, lang=None):
        '''
        modify an existing line
        '''
        if not lang:
            editlang = self.editlang
        else:
            editlang = str(lang)
        self.checklang(editlang)
        if (self.isValidLine(line,editlang)):
            self.lines[editlang][line]=text
            self.cachere[editlang]=True
        else:
            raise IndexError("Popuplib: Cannot modify line #%d in %s"%(line,self.name))
    def addlineAll(self, text):
        for lang in self.lines:
            self.addline(text, lang)
    def dellineAll(self, line):
        for lang in self.lines:
            try:
                self.delline(line, lang)
            except IndexError:
                pass
    def inslineAll(self, line, text):
        for lang in self.lines:
            try:
                self.insline(line, text, lang)
            except IndexError:
                pass
    def modlineAll(self, line, text):
        for lang in self.lines:
            try:
                self.modline(line, text, lang)
            except IndexError:
                pass
    def select(self, item, block):
        '''
        set a select block
        '''
        if (self.isValidItem(item)): self.p_select[item]=block
        else:
            raise IndexError("Popuplib: Invalid menu item %d for select in %s"%(item,self.name))
    def submenu(self, item, menuname):
        '''
        set a submenu
        '''
        if (self.isValidItem(item)): self.p_submenu[item]=menuname
        else:
            raise IndexError("Popuplib: Invalid menu item %d for submenu in %s"%(item,self.name))
    def menuvalue(self, varn, item, varv):
        '''
        set menuvalue
        '''
        if not isinstance(item,int):
            a = str(item)
            item = int(varn)
            varn = a
        if (self.isValidItem(item)):
            self.menuvar[item]=varn
            self.menuval[item]=varv
        else:
            raise IndexError("Popuplib: Invalid menu item %d for menuvalue in %s"%(item,self.name))
    def recache(self, users=[]):
        '''
        recache the popup
        '''
        es.dbgmsg(1,"Popuplib: recache(users="+str(users)+str(type(users))+")")
        try:
            user = int(users)
            userlang = self.i_userlang(user)
            if userlang:
                self.cache[userlang] = self.i_recache(userlang)
                self.cachere[userlang] = False
                gUsers[user].cache[self.name]=self.cache[userlang]
        except TypeError:
            cachedlangs = []
            for user in users:
                userlang = self.i_userlang(int(user))
                if userlang:
                    if not userlang in cachedlangs:
                        self.cache[userlang] = self.i_recache(userlang)
                        cachedlangs.append(userlang)
                    gUsers[int(user)].cache[self.name]=self.cache[userlang]
    def i_recache(self, userlang):
        '''
        INTERNAL
        do the actual construction
        '''
        es.dbgmsg(1,"Popuplib: i_recache(%s)"%userlang)
        build = ""
        for line in self.lines[userlang]:
            build += self.lines[userlang][line] + "\\n"   #need double \\ for python uses \n internally also
        if len(build) > 0:
            build = build[0:-2]         #remove the last \n
        es.dbgmsg(1,'Popuplib: i_recache returns "'+build+'"')
        return build
    def checkCache(self, userid):
        '''
        INTERNAL (semi)
        check the cache status and recache as necessary
        '''
        es.dbgmsg(1,"Popuplib: checkCache(userid="+str(userid)+str(type(userid))+")")
        userlang = self.i_userlang(int(userid))
        if not userlang:
            return "Empty popup!"
        es.dbgmsg(1,'Popuplib: userlang="%s"'%userlang)
        es.dbgmsg(1,'Popuplib: cachemode="'+str(self.cachemode)+str(type(self.cachemode))+'"')
        es.dbgmsg(1,'Popuplib: cachere="'+str(self.cachere[userlang])+str(type(self.cachere[userlang]))+'"')
        if self.cachemode == "global" and self.cachere[userlang]:
            es.dbgmsg(1,'Popuplib: Issuing recache on global popup language %s'%userlang)
            self.recache(userid)
        if self.cachemode != "user":
            es.dbgmsg(1,'Popuplib: Not user-specific, returning cache="'+str(self.cache[userlang])+str(type(self.cache[userlang]))+'"')
            return self.cache[userlang]
        else:
            es.dbgmsg(1,'Popuplib: User specific popup')
            if not (self.name in gUsers[userid].cache):
                es.dbgmsg(1,'Popuplib: Popup not in user cache, recaching')
                self.recache(userid)
            return gUsers[userid].cache[self.name]
    
    def handleChoice(self, choice, userid):
        '''
        handle menu choice as a result of client command
        '''
        es.dbgmsg(1, "Popuplib: popup.handleChoice ("+str(choice)+str(type(choice))+", "+str(userid)+str(type(userid))+")")
        user=gUsers[userid]     # retrieve user object
        es.dbgmsg(1, "Popuplib: displaymode='"+str(self.displaymode)+"'"+str(type(self.displaymode)))
        activeid = user.activeid
        # check displaymode initially
        if self.displaymode == "sticky":
            issticky=True
        else:
            issticky=False
        
        #set server variables for non-python scripts
        es.set('_popup_userid',userid,"User of the popup")
        es.set('_popup_choice',choice,"Menu item chosen")
        es.set('_popup_name',self.name,"Active menu name")

        #check for menuvalue
        if choice in self.menuvar:
            es.set(self.menuvar[choice],self.menuval[choice])

        #check for select block
        pselect = None
        if choice in self.p_select:
            pselect = self.p_select[choice]
            es.dbgmsg(1, "Popuplib: Select found")
            es.dbgmsg(1, "Popuplib: select block='"+str(pselect)+"'"+str(type(pselect)))
            if callable(pselect):
                pselect(userid,choice,self.name)
            else:
                es.doblock(pselect)
            issticky=False

        #check for menuselect block
        if self.menuselect != "":
            es.dbgmsg(1, "Popuplib: menuselect='"+str(self.menuselect)+"'"+str(type(self.menuselect)))
            if callable(self.menuselect):
                self.menuselect(userid, choice,self.name)
            else:
                es.doblock(self.menuselect)

        #check for menuselectfb block if select block was not used
        if not pselect and self.menuselectfb != "":
            es.dbgmsg(1, "Popuplib: menuselectfb='"+str(self.menuselectfb)+"'"+str(type(self.menuselectfb)))
            if callable(self.menuselectfb):
                self.menuselectfb(userid, choice,self.name)
            else:
                es.doblock(self.menuselectfb)

        #check for submenu
        if choice in self.p_submenu:
            psubmenu = self.p_submenu[choice]
            es.dbgmsg(1, "Popuplib: Submenu found")
            es.dbgmsg(1, "Popuplib: submenu='"+str(psubmenu)+"'"+str(type(psubmenu)))
            if isinstance(psubmenu, Popup_base):
                user.queue[activeid] = psubmenu
            else:
                user.queue[activeid] = find(psubmenu)
            issticky=False

        #check if the popup was sticky and no custom thing was set to menu item
        if issticky:
            es.dbgmsg(1, "Popuplib: Sending sticky key")
            es.cexec(userid, "slot"+str(choice))
            es.delayed(.2, 'es_xmenu 0 '+str(userid)+' "'+self.checkCache(userid)+'"')
        else:
            #popup finished, fire event and process user queue
            es.event("initialize","popup_select")
            es.event("setint","popup_select","userid",userid)
            es.event("setstring","popup_select","popup_name",self.name)
            es.event("setint","popup_select","popup_choice",choice)
            es.event("fire","popup_select")
            if user.active == self:
                es.dbgmsg(1, "Popuplib: Redoing user queue")
                user.active = None
                user.handleQueue()
            else:
                del user.queue[activeid]
                if user.active:
                    user.updatePopup(user.active)

class Popup_construct(Popup_base):
    '''
    Construct type popup
    '''
    def __init__(self, pPopupid, pType, pFlags):
        '''
        Initialize the new popup by creating all necessary attributes
        with their default values
        '''
        Popup_base.__init__(self, pPopupid)
        self.ctype = pType
        self.flags = pFlags
        self.c_savevar = "_popup_choice"
        self.description = ""
        self.c_stateby = "True"
        self.c_stateformat = {}
        self.c_stateformat[False] = "%1. %n"
        self.c_stateformat[True] = "->%1. %n"
        self.c_titleformat = _pnformat(pPopupid,30)
        self.c_beginsep = "-----------------------------"
        self.c_pagesep = "-----------------------------"
        self.c_endsep = ""
        self.c_pageformat = "->%1. %2"
        self.c_exitformat = "0. %2"
        #self.c_language = gLanguage
        self.c_gamename = gGamename
        self.p_submenu = None
        self.menuselect = ""
        self.menuselectfb = ""
        self.cachemode = "eval"
        self.cache = {1: "Uncached construct %s"%self.name}
        self.type = "construct"
        self.constructlist = None
        self.activepage = {}
        self.varvals = []
    def settitle(self, desc, pagenumbers=30, lang=None):
        '''
        Set the title
        '''
        if isinstance(desc, dict):
            # Language object
            self.c_titleformat = _pnformat(str(desc[gLanguage]), int(pagenumbers))
        else:
            self.c_titleformat[editlang] = _pnformat(str(desc), int(pagenumbers))
    def information(self, listlevel):
        '''
        Print information about this popup to server console
        '''
        es.dbgmsg(0, self.name)
        if listlevel >= 1:
            es.dbgmsg(0, "  Type:         "+str(self.type))
            es.dbgmsg(0, "  List:         "+str(self.ctype))
            es.dbgmsg(0, "  Flags:        "+str(self.flags))
            es.dbgmsg(0, "  Menuselect:   "+str(self.menuselect))
            es.dbgmsg(0, "  Menuselectfb: "+str(self.menuselectfb))
    def submenu(self, item, menuname=None):
        '''
        set a submenu for menu 10
        '''
        if not menuname:
            menuname = item
        self.p_submenu = menuname
    def recache(self, users=[]):
        '''
        recache the popup
        '''
        es.dbgmsg(1,"Popuplib: recache(users="+str(users)+str(type(users))+")")
        self.i_recache()
        try:
            user = gUsers[int(users)]
            if user: user.cache[self.name]=self.cache
        except TypeError:
            for userid in users:
                user = gUsers[int(userid)]
                if user: user.cache[self.name]=self.cache
    def i_recache(self):
        '''
        INTERNAL
        do the actual construction
        '''
        es.dbgmsg(1,"Popuplib: "+self.name+" construct i_recache()")
        self.cache = {}
        self.varvals = []
        if self.ctype == "players":
            es.dbgmsg(1,"Popuplib: playerlist "+self.flags)
            self.constructlist = playerlib.getPlayerList(self.flags)
            players = len(self.constructlist)
            if players <= 9:
                totalpages = 1
            else:
                totalpages = int(math.ceil(float(players)/7))
            page = 1
            build = self.pageformatstart(page,totalpages)
            line = 0
            for pl in self.constructlist:
                line += 1
                playerid = int(pl)
                self.varvals.append(playerid)
                thisformat = str(self.c_stateformat[bool(eval(self.c_stateby))])
                thisformat = thisformat.replace("%1", str(line))
                thisformat = thisformat.replace("%t", str(pl.attributes['teamid']))
                thisformat = thisformat.replace("%T", str(teamdata[self.c_gamename][int(pl.attributes['teamid'])]))
                thisformat = thisformat.replace("%n", str(pl.attributes['name']))
                thisformat = thisformat.replace("%u", str(playerid))
                thisformat = thisformat.replace("%B", str(cbotdata[pl.attributes['isbot']]))
                thisformat = thisformat.replace("%D", str(cdeaddata[pl.attributes['isdead']]))
                build += thisformat + "\\n"
                if line == 7 and totalpages > 1:
                    #build += self.pageformatend(page, totalpages)
                    self.cache[page] = build
                    page += 1
                    line = 0
                    build = self.pageformatstart(page,totalpages)
            if totalpages > 1:
                if line != 0:
                    for emptyline in range(line,7):
                        build += " \\n"
                    #build += self.pageformatend(page,totalpages)
                    self.cache[page] = build
            else:
                for emptyline in range(line,9):
                    build += " \\n"
                #build += self.pageformatend(page,totalpages,True)
                self.cache[page] = build
                
    def pageformatstart(self, page, total):
        '''
        INTERNAL
        Format the start of the page
        '''
        build = str(self.c_titleformat).replace("%p", str(page))
        build = build.replace("%t", str(total)) + "\\n"
        if self.description:
            build += str(self.description) + "\\n"
        if self.c_beginsep:
            build += str(self.c_beginsep) + "\\n"
        return build
    def pageformatend(self,page,total,language):
        '''
        INTERNAL
        Format the end of the page
        '''
        if total > 1:
            tight = False
        else:
            tight = True
        build = ""
        if self.c_pagesep:
            build += str(self.c_pagesep) + "\\n"
        if page > 1: build += str(self.c_pageformat).replace("%1", "8").replace("%2", langdata[language]['prev'])+"\\n"
        elif not tight: build += " \\n"
        if page < total: build += str(self.c_pageformat).replace("%1", "9").replace("%2", langdata[language]['next'])+"\\n"
        elif not tight: build += " \\n"
        if self.c_endsep:
            build += str(self.c_endsep) + "\\n"
        build += str(self.c_exitformat).replace("%1", "10").replace("%2", langdata[language]['quit'])
        return build
    def sendPage(self, users, pagenumber):
        '''
        Set the active page for the users and send the popup
        '''
        if pagenumber:
            try:
                userid = int(users)
                userlist = [userid]
            except TypeError:
                userlist = list(users)
            for userid in userlist:
                self.activepage[int(userid)]=pagenumber
        self.send(users)
    def getPage(self, userid):
        '''
        Returns the active page number of selected user
        '''
        return self.activepage[int(userid)]
    def unsendPage(self, users, pagenumber):
        '''
        Unsends this popup from users if they are on specific page
        '''
        try:
            userid = int(users)
            userlist = [userid]
        except TypeError:
            userlist = list(users)
        for userid in userlist:
            if self.activepage[int(userid)]==pagenumber:
                self.unsend(userid)
    def getPageCount(self):
        '''
        Returns the number of pages in this popup
        '''
        return len(self.cache)
    
    def checkCache(self, userid):
        '''
        INTERNAL (semi)
        check the cache status and recache as necessary
        '''
        if not gUsers[int(userid)]:
            return None
        allowrecache = True
        if int(userid) in self.activepage:
            page = self.activepage[int(userid)]
            allowrecache = False
        else:
            page = 1
            self.activepage[int(userid)] = 1
        es.dbgmsg(1,"Popuplib: checkCache(userid="+str(userid)+str(type(userid))+")")
        es.dbgmsg(1,"Popuplib: checkCache(page="+str(page)+str(type(page))+")")
        es.dbgmsg(1,'Popuplib: cachemode="'+str(self.cachemode)+str(type(self.cachemode))+'"')
        if allowrecache and self.cachemode == "eval":
            if self.ctype == "players":
                es.dbgmsg(1,"Popuplib: playerlist "+self.flags)
                comp_constructlist = playerlib.getPlayerList(self.flags)
            if comp_constructlist != self.constructlist:
                es.dbgmsg(1,"Popuplib: recache")
                self.recache(userid)
        pages = len(self.cache)
        if page < 1:
            page = 1
        elif page > pages:
            page = pages
        self.activepage[int(userid)] = page
        if self.cachemode != "user":
            es.dbgmsg(1,'Popuplib: Not user-specific, returning cache="'+str(self.cache[int(page)])+str(type(self.cache[int(page)]))+'"')
            return self.cache[int(page)]+self.pageformatend(page,pages,self.i_endlang(int(userid)))
        else:
            es.dbgmsg(1,'Popuplib: User specific popup')
            if not (self.name in gUsers[userid].cache):
                es.dbgmsg(1,'Popuplib: Popup not in user cache, recaching')
                self.recache(userid)
            return gUsers[userid].cache[self.name][int(page)]+self.pageformatend(page,pages,self.i_endlang(int(userid)))
    
    def handleChoice(self, choice, userid):
        '''
        handle menu choice as a result of client command
        '''
        es.dbgmsg(1, "Popuplib: construct.handleChoice ("+str(choice)+str(type(choice))+", "+str(userid)+str(type(userid))+")")
        user=gUsers[userid]     # retrieve user object
        activeid = user.activeid
        es.dbgmsg(1, "Popuplib: construct='"+str(self.ctype)+"'"+str(type(self.ctype)))
        es.dbgmsg(1, "Popuplib: flags='"+str(self.flags)+"'"+str(type(self.flags)))
        page = self.activepage[int(userid)]
        pages = len(self.cache)
        choices = len(self.varvals)
        varvalindex = 7*(page-1)+choice-1
        validchoice = False
        
        #set server variables for non-python scripts
        es.set('_popup_userid',userid,"User of the popup")
        es.set('_popup_choice',choice,"Menu item chosen")
        es.set('_popup_name',self.name,"Active menu name")
        if varvalindex < choices:
            choiceval = self.varvals[varvalindex]
            if pages > 1 and choice < 8:
                validchoice = True
            elif pages == 1 and choice < 10:
                validchoice = True
        if not validchoice:
            choiceval = choice
        else:
            es.set(self.c_savevar,choiceval,"Chosen object")
            del self.activepage[int(userid)]

        #check for menuselect block
        if self.menuselect != "":
            es.dbgmsg(1, "Popuplib: menuselect='"+str(self.menuselect)+"'"+str(type(self.menuselect)))
            if callable(self.menuselect):
                self.menuselect(userid, choiceval,self.name)
            else:
                es.doblock(self.menuselect)

        #check for menuselectfb block if valid user
        if validchoice and self.menuselectfb != "":
            es.dbgmsg(1, "Popuplib: menuselectfb='"+str(self.menuselectfb)+"'"+str(type(self.menuselectfb)))
            if callable(self.menuselectfb):
                self.menuselectfb(userid, choiceval,self.name)
            else:
                es.doblock(self.menuselectfb)

        #check for submenu (=send popup again and change page if applicable)
        if not validchoice and choice < 10:
            if pages > 1:
                if choice > 7:
                    if choice == 8:
                        page -= 1
                        if page == 0:
                            page = 1
                    else:
                        page += 1
                        if page > pages:
                            page = pages
                    self.activepage[int(userid)] = page
            user.queue[activeid] = self
        elif choice == 10:
            del self.activepage[int(userid)]
            if self.p_submenu:
                if isinstance(self.p_submenu, Popup_base):
                    user.queue[activeid] = self.p_submenu
                else:
                    user.queue[activeid] = find(self.p_submenu)

        #popup finished, fire event and process user queue
        es.event("initialize","popup_select")
        es.event("setint","popup_select","userid",userid)
        es.event("setstring","popup_select","popup_name",self.name)
        es.event("setint","popup_select","popup_choice",choice)
        es.event("fire","popup_select")
        if user.active == self:
            es.dbgmsg(1, "Popuplib: Redoing user queue")
            user.active = None
            user.handleQueue()
        else:
            del user.queue[activeid]
            if user.active:
                user.updatePopup(user.active)

class Easymenu_option:
    '''
    A helper class for easymenu options
    '''
    def __init__(self, item, text, state, index):
        self.value = item
        self.text = str(text)
        self.state = state
        self.index = index

class Popup_easymenu(Popup_base):
    '''
    Popup easymenu allowing easy construction of multi-page menus
    '''
    def __init__(self, pPopupid, pVar, pBlock):
        '''
        Initialize the popup by creating necessary attributes
        with their default values
        '''
        Popup_base.__init__(self, pPopupid)
        self.options = {}
        self.c_savevar = pVar
        self.descriptiond = {}
        self.c_stateformat = {}
        self.c_stateformat[False] = "%1. %2"
        self.c_stateformat[True] = "->%1. %2"
        self.c_titleformat = _pnformat(pPopupid,30)
        self.l_titleformat = {}
        self.c_beginsep = "-----------------------------"
        self.c_pagesep = "-----------------------------"
        self.c_endsep = ""
        self.c_pageformat = "->%1. %2"
        self.c_exitformat = "0. %2"
        #self.c_language = gLanguage
        self.p_submenu = None
        self.menuselect = ""
        self.menuselectfb = pBlock
        self.cachemode = "easymenu"
        self.cache = {}
        self.cachere = {}
        self.type = "easymenu"
        self.activepage = {}
        self.checklang()
    def setdescription(self, desc, lang=None):
        '''
        Set the description for the easymenu
        '''
        if isinstance(desc, dict):
            # Language object
            for editlang in desc:
                self.descriptiond[editlang] = str(desc[editlang])
        else:
            if not lang:
                editlang = self.editlang
            else:
                editlang = str(lang)
            self.descriptiond[editlang] = str(desc)
    def settitle(self, desc, pagenumbers=30, lang=None):
        '''
        Set the title for the easymenu
        '''

        if isinstance(desc, dict):
            # Language object
            for editlang in desc:
                self.l_titleformat[editlang] = _pnformat(str(desc[editlang]), int(pagenumbers))
        else:
            if not lang:
                editlang = self.editlang
            else:
                editlang = str(lang)
            self.l_titleformat[editlang] = _pnformat(str(desc), int(pagenumbers))
    def checklang(self, lang=None):
        '''
        INTERNAL
        Checks if the current language exists and create it as necessary
        '''
        if not lang:
            editlang = self.editlang
        else:
            editlang = str(lang)
        if not self.options.has_key(editlang):
            self.options[editlang] = {}
            self.cache[editlang] = {}
            self.cachere[editlang] = {1:True}
            self.descriptiond[editlang] = ""
            if editlang not in self.l_titleformat:
                self.l_titleformat[editlang] = str(self.c_titleformat)
    
    def information(self, listlevel):
        '''
        Print out some information to server console
        '''
        es.dbgmsg(0, self.name)
        if listlevel >= 1:
            es.dbgmsg(0, "  Type:         "+str(self.type))
            es.dbgmsg(0, "  Variable:     "+str(self.c_savevar))
            es.dbgmsg(0, "  Options:      "+str(len(self.options)))
            es.dbgmsg(0, "  Menuselect:   "+str(self.menuselect))
            es.dbgmsg(0, "  Menuselectfb: "+str(self.menuselectfb))
    
    def addoption(self, item, text, state=1, lang=None):
        '''
        Add a new option to the menu
        '''
        if isinstance(text, dict):
            # Assume as dict-type object with language names as keys
            es.dbgmsg(1, "Popuplib.py: addoption detected language object")
            for editlang in text:
                self.checklang(editlang)
                index = len(self.options[editlang])+1
                self.options[editlang][index]=Easymenu_option(item, text[editlang], state, index)
                self.cachere[editlang][self.getPageNumber(len(self.options[editlang]), editlang)]=True
                if index == 10:
                    self.cachere[editlang][1] = True
                    # first page needs caching also when moving to page 2
        else:
            es.dbgmsg(1, "Popuplib.py: addoption detected text object")
            if not lang:
                editlang = self.editlang
            else:
                editlang = str(lang)
            self.checklang(editlang)
            index = len(self.options[editlang])+1
            self.options[editlang][index]=Easymenu_option(item, text, state, index)
            self.cachere[editlang][self.getPageNumber(len(self.options[editlang]), editlang)]=True
            if index == 10:
                self.cachere[editlang][1] = True
                # first page needs caching also when moving to page 2
    
    def findOption(self, item, lang=None):
        '''
        INTERNAL
        Find option by its value
        '''
        if not lang:
            editlang = self.editlang
        else:
            editlang = str(lang)
        self.checklang(editlang)
        for i in self.options[editlang]:
            opt = self.options[editlang][i]
            if opt.value == item: return opt
        return None
    def setoption(self, item, text=None, state=None, lang=None):
        '''
        Edit an existing option
        '''
        if not lang:
            editlang = self.editlang
        else:
            editlang = str(lang)
        self.checklang(editlang)
        opt = self.findOption(item, editlang)
        if opt:
            self.cachere[editlang][self.getPageNumber(opt.index, editlang)]=True
            if text:
                opt.text = text
            if state:
                opt.state = state
        else:
            raise IndexError("Popuplib: Option %s does not exist in easymenu %s"%(item,self.name))
    def getPageNumber(self, option, language):
        '''
        INTERNAL
        Returns the page number on which the option is present in
        '''
        opts = len(self.options[language])
        if opts < 10: return 1
        return int(math.ceil(float(option)/7))
    def submenu(self, item, menuname=None):
        '''
        set a submenu for menuselect 10
        '''
        if not menuname:
            menuname = item
        self.p_submenu = menuname
    def recache(self, users=[], pages=None):
        '''
        recache the popup
        '''
        es.dbgmsg(1,"Popuplib: recache(users="+str(users)+str(type(users))+")")
        try:
            user = int(users)
            userlang = self.i_userlang(user)
            es.dbgmsg(1,"Popuplib: compatible user language: %s"%str(userlang))
            if userlang:
                if not pages:
                    pages = range(1,self.getPageNumber(len(self.options[userlang]))+1)
                self.i_recache(pages, userlang)
                gUsers[user].cache[self.name]=self.cache[userlang]
        except TypeError:
            cachedlangs = []
            for user in users:
                userlang = self.i_userlang(int(user))
                es.dbgmsg(1,"Popuplib: compatible user %d language: %s"%(int(user),str(userlang)))
                if userlang:
                    if not userlang in cachedlangs:
                        if not pages:
                            pages = range(1,self.getPageNumber(len(self.options[userlang]))+1)
                        self.cache[userlang] = self.i_recache(pages, userlang)
                        cachedlangs.append(userlang)
                    gUsers[int(user)].cache[self.name]=self.cache[userlang]
    def i_recache(self, pages, userlang):
        '''
        INTERNAL
        do the actual construction
        '''
        es.dbgmsg(1,"Popuplib: %s easymenu i_recache(%s)"%(self.name,userlang))
        maxindex = len(self.options[userlang])
        totalpages = self.getPageNumber(maxindex, userlang)
        if totalpages == 1:
            maxline=9
        else:
            maxline=7
        for page in pages:
            build = self.pageformatstart(page,totalpages,userlang)
            for line in range(1,maxline+1):
                index = (page-1)*7+line
                if index <= maxindex:
                    opt = self.options[userlang][index]
                    thisformat = str(self.c_stateformat[bool(opt.state)])
                    thisformat = thisformat.replace("%1", str(line)).replace("%2", str(opt.text))
                    build += thisformat + "\\n"
                else:
                    build += " \\n"
            build += self.pageformatend(page,totalpages,self.i_endlangg(userlang))
            self.cache[userlang][page] = build
            self.cachere[userlang][page] = False
                
    def pageformatstart(self, page, total, language):
        '''
        INTERNAL
        Formats the start of a page
        '''
        es.dbgmsg(1,"Popuplib: pageformatstart for page %d/%d in lang %s"%(page,total,language))
        build = str(self.l_titleformat[language]).replace("%p", str(page))
        build = build.replace("%t", str(total)) + "\\n"
        if self.descriptiond[language]:
            build += str(self.descriptiond[language]) + "\\n"
        if self.c_beginsep:
            build += str(self.c_beginsep) + "\\n"
        return build
    def pageformatend(self, page, total, language):
        '''
        INTERNAL
        Formats the end of a page
        '''
        if total > 1:
            tight = False
        else:
            tight = True
        build = ""
        if self.c_pagesep:
            build += str(self.c_pagesep) + "\\n"
        if page > 1: build += str(self.c_pageformat).replace("%1", "8").replace("%2", langdata[language]['prev'])+"\\n"
        elif not tight: build += " \\n"
        if page < total: build += str(self.c_pageformat).replace("%1", "9").replace("%2", langdata[language]['next'])+"\\n"
        elif not tight: build += " \\n"
        if self.c_endsep:
            build += str(self.c_endsep) + "\\n"
        build += str(self.c_exitformat).replace("%1", "10").replace("%2", langdata[language]['quit'])
        return build
    def sendPage(self, users, pagenumber):
        '''
        Sets active page and sends the menu to users
        '''
        if pagenumber:
            try:
                userid = int(users)
                userlist = [userid]
            except TypeError:
                userlist = list(users)
            for userid in userlist:
                self.activepage[int(userid)]=pagenumber
        self.send(users)
    def getPage(self, userid):
        '''
        Returns the page number the user has active
        '''
        return self.activepage[int(userid)]
    def unsendPage(self, users, pagenumber):
        '''
        Unsends the easymenu from users that have the specified page active
        '''
        try:
            userid = int(users)
            userlist = [userid]
        except TypeError:
            userlist = list(users)
        for userid in userlist:
            if self.activepage[int(userid)]==pagenumber:
                self.unsend(userid)
    def getPageCount(self, userlang=None):
        '''
        Returns the number of pages
        '''
        if not userlang: userlang = self.editlang
        return self.getPageNumber(len(self.options[userlang]), userlang)
        
    def checkCache(self, userid):
        '''
        INTERNAL (semi)
        check the cache status and recache as necessary
        '''
        if not gUsers[int(userid)]:
            return None
        if int(userid) in self.activepage:
            page = self.activepage[int(userid)]
        else:
            page = 1
            self.activepage[int(userid)] = 1
        es.dbgmsg(1,"Popuplib: checkCache(userid="+str(userid)+str(type(userid))+")")
        es.dbgmsg(1,"Popuplib: checkCache(page="+str(page)+str(type(page))+")")
        es.dbgmsg(1,'Popuplib: cachemode="'+str(self.cachemode)+str(type(self.cachemode))+'"')
        userlang = self.i_userlang(int(userid))
        if self.cachemode == "easymenu" and self.cachere[userlang][page]:
            es.dbgmsg(1,"Popuplib: recache")
            self.recache(userid,[page])
        pages = self.getPageNumber(len(self.options[userlang]), userlang)
        if page < 1:
            page = 1
        elif page > pages:
            page = pages
        self.activepage[int(userid)] = page
        if self.cachemode != "user":
            es.dbgmsg(1,'Popuplib: Not user-specific, returning cache="'+str(self.cache[userlang][int(page)])+str(type(self.cache[userlang][int(page)]))+'"')
            return self.cache[userlang][int(page)]
        else:
            es.dbgmsg(1,'Popuplib: User specific popup')
            if not (self.name in gUsers[userid].cache):
                es.dbgmsg(1,'Popuplib: Popup not in user cache, recaching')
                self.recache(userid,[page])
            return gUsers[userid].cache[self.name][int(page)]
    
    def handleChoice(self, choice, userid):
        '''
        handle menu choice as a result of client command
        '''
        es.dbgmsg(1, "Popuplib: easymenu.handleChoice ("+str(choice)+str(type(choice))+", "+str(userid)+str(type(userid))+")")
        userlang = self.i_userlang(int(userid))
        user=gUsers[userid]     # retrieve user object
        activeid = user.activeid
        page = self.activepage[int(userid)]
        pages = self.getPageNumber(len(self.options[userlang]), userlang)
        es.dbgmsg(1, "Popuplib: page "+str(page)+"/"+str(pages))
        choices = len(self.options[userlang])
        es.dbgmsg(1, "Popuplib: choice "+str(choice)+"/"+str(choices))
        varvalindex = 7*(page-1)+choice
        es.dbgmsg(1, "Popuplib: index="+str(varvalindex))
        validchoice = False
        
        #set server variables for non-python scripts
        es.set('_popup_userid',userid,"User of the popup")
        es.set('_popup_choice',choice,"Menu item chosen")
        es.set('_popup_name',self.name,"Active menu name")
        if varvalindex <= choices:
            opt = self.options[userlang][varvalindex]
            if opt.state:
                choiceval = opt.value
                if pages > 1 and choice < 8:
                    validchoice = True
                elif pages == 1 and choice < 10:
                    validchoice = True
        if not validchoice:
            choiceval = choice
        else:
            es.set(self.c_savevar,str(choiceval),"Chosen object")
            del self.activepage[int(userid)]

        #check for menuselect block
        if self.menuselect != "":
            es.dbgmsg(1, "Popuplib: menuselect='"+str(self.menuselect)+"'"+str(type(self.menuselect)))
            if callable(self.menuselect):
                self.menuselect(userid, choiceval, self.name)
            else:
                es.doblock(self.menuselect)

        #check for menuselectfb block if valid user
        if validchoice and self.menuselectfb != "":
            es.dbgmsg(1, "Popuplib: menuselectfb='"+str(self.menuselectfb)+"'"+str(type(self.menuselectfb)))
            if callable(self.menuselectfb):
                self.menuselectfb(userid, choiceval,self.name)
            else:
                es.doblock(self.menuselectfb)

        #check for submenu (=send popup again and change page if applicable)
        if not validchoice and choice < 10:
            if pages > 1:
                if choice > 7:
                    if choice == 8:
                        page -= 1
                        if page == 0:
                            page = 1
                    else:
                        page += 1
                        if page > pages:
                            page = pages
                    self.activepage[int(userid)] = page
            user.queue[activeid] = self
        elif choice == 10:
            del self.activepage[int(userid)]
            if self.p_submenu:
                if isinstance(self.p_submenu, Popup_base):
                    user.queue[activeid] = self.p_submenu
                else:
                    user.queue[activeid] = find(self.p_submenu)

        #popup finished, fire event and process user queue
        es.event("initialize","popup_select")
        es.event("setint","popup_select","userid",userid)
        es.event("setstring","popup_select","popup_name",self.name)
        es.event("setint","popup_select","popup_choice",choice)
        es.event("fire","popup_select")
        if user.active == self:
            es.dbgmsg(1, "Popuplib: Redoing user queue")
            user.active = None
            user.handleQueue()
        else:
            del user.queue[activeid]
            if user.active:
                user.updatePopup(user.active)


class Popup_user:
    '''
    INTERNAL class
    that stores information about a player, mainly their popup queue
    '''
    def __init__(self, userid):
        '''
        Initialize the player info
        '''
        self.userid = userid        #save the userid
        self.queue = {}             #empty popup queue
        self.active = None          #no active popup currently
        self.activeid = 0           #active queue id
        self.nextindex = 1          #next queue index
        self.cache = {}             #popup cache
        self.prioritized = False    #used for queue priorization
        self.language = gLanguage   #user language
        self.updateLanguage()
        self.closemode = False      #close mode
    
    def updateLanguage(self):
        '''
        Update the language info
        '''
        userobject = playerlib.getPlayer(self.userid)
        language = userobject.get('lang')
        es.dbgmsg(1,'Popuplib: User %d language="%s"'%(self.userid, language))
        if language: self.language=language
        
    def handleQueue(self):
        '''
        Display the next popup in player's queue!!!11!!1
        '''
        es.dbgmsg(1, "Popuplib: User.handleQueue(userid:"+str(self.userid)+")")
        es.dbgmsg(1, "Popuplib: activeid="+str(self.activeid)+str(type(self.activeid)))
        es.dbgmsg(1, "Popuplib: queue="+str(self.queue)+"("+str(len(self.queue))+" items)")
        if self.active == None and len(self.queue) > 0:
            #player doesn't have active popup but has some in queue = display next
            gUsers.haspopup(self)
            qindex = min(self.queue)
            qpopup = self.queue[qindex]
            es.dbgmsg(1, "Popuplib: qindex='"+str(qindex)+"'"+str(type(qindex)))
            es.dbgmsg(1, "Popuplib: qpopup='"+str(qpopup)+"'"+str(type(qpopup)))
            del self.queue[qindex]      #delete popup from queue
            es.dbgmsg(1, "Popuplib: queue after="+str(self.queue)+"("+str(len(self.queue))+" items)")
            prepuser = qpopup.prepuser
            es.dbgmsg(1, "Popuplib: prepuser='"+str(prepuser)+"'"+str(type(prepuser)))
            if callable(prepuser):
                #python function for prepuser :D
                prepuser(self.userid, qpopup.name)
            elif prepuser != "":
                #regular es block for prepuser
                es.set('_popup_userid',self.userid)
                es.set('_popup_name',qpopup.name)
                #es.doblock(prepuser) <- does not work :(
                es.server.cmd("es_xdoblock "+str(prepuser))
                #the popup might be recreated in the prepuser block, reload it:
                qpopup = gPopups[qpopup.name]

            #get the popup properties and display it to user
            menudisplay = qpopup.checkCache(self.userid)
            pendingcount = len(self.queue)
            if pendingcount > 0:
                pendingtext = "\\n" + ("-"*29)+"\\n ("+langdata[gLanguage]['pending'] + ": "+str(pendingcount)+") "
            else:
                pendingtext = ""
            menudisplay += pendingtext
            es.dbgmsg(1, "Popuplib: menudisplay='"+str(menudisplay)+"'"+str(type(menudisplay)))
            timeout = qpopup.timeoutview
            es.menu(timeout, self.userid, menudisplay)

            #set active popup info
            self.active = qpopup
            self.activeid = qindex

            #perform timeout check
            if timeout > 0:
                gamethread.delayed(timeout, unsendid, (qindex, self.userid))

            #fire event popup_display
            es.event("initialize","popup_display")
            es.event("setint","popup_display","userid",self.userid)
            es.event("setstring","popup_display","popup_name",qpopup.name)
            es.event("setfloat","popup_display","popup_timeout",timeout)
            es.event("fire","popup_display")
        elif len(self.queue) > 0:
            gUsers.haspopup(self)
            self.updatePopup(self.active)
        elif self.active == None:
            gUsers.hasnopopup(self)
    
    def unsendView(self, index):
        '''
        remove popup from display if the queue index matches
        '''
        if index == self.activeid:
            if self.active.oldskooltov:
                #some older scripts want to receive menuselect 10
                self.active.handleChoice(10, self.userid)
            else:
                #get next popup from queue, if there is one
                self.active = None
                self.handleQueue()
    def unsendSend(self, index):
        '''
        remove popup from queue by queue index
        '''
        if index in self.queue:
            del self.queue[index]
            #update pending counter:
            if self.active:
                self.queue[self.activeid] = self.active
                self.handleQueue()
    
    def unsendName(self, popupid, close=False):
        '''
        remove popups from queue by the name
        '''
        compopup = gPopups[popupid]
        self.unsend(compopup, close)
        
    def unsend(self, compopup, close):
        '''
        remove popups from queue by the class handler
        '''

        # since you cannot delete or add items from dictionary in a loop,
        # create temporary list of the queue indexes for the loop:
        tempqueue = list(self.queue)
        for index in tempqueue:
            if compopup == self.queue[index]:
                del self.queue[index]
        if self.active == compopup:
            #the active one mathced also!
            if close:
                #close mode used, don't process current popup
                self.active = None
                self.closemode = True
                if len(self.queue) > 0:
                    self.handleQueue()
                else:
                    es.cexec(self.userid,"slot10")
            else:
                #close the popup
                es.cexec(self.userid,"slot10")
    
    def isqueued(self, p):
        '''
        return first queue position of the popup
        '''
        if self.active == p: return 1
        go = 1
        if self.queue:
            for index in range(min(self.queue),max(self.queue)+1):
                if index in self.queue:
                    go += 1
                    if self.queue[index] == p:
                        return go
        return 0
    
    def updatePopup(self, p):
        '''
        redisplay the popup if its active
        '''
        es.dbgmsg(1,"Popuplib: Userid"+str(self.userid)+".updatePopup("+str(p)+str(type(p))+")")
        es.dbgmsg(1,"Popuplib: active = "+str(self.active)+str(type(self.active)))
        if self.active is p:
            es.dbgmsg(1,"Popuplib: match!")
            self.queue[self.activeid] = p
            self.active = None
            self.handleQueue()

class UserHolderClass(object):
    '''
    This is an internal class for handling user objects
    '''
    def __init__(self):
        '''
        Create attributes and steal players from server
        '''
        self.userdict = {}
        self.activepopups = set()
        userlist = playerlib.getUseridList("#human")
        for userid in userlist:
            self[int(userid)] = Popup_user(int(userid))
        es.addons.unregisterForEvent(selfmodule, 'es_client_command')
    def __setitem__(self, userid, value):
        '''
        x.__setitem__(userid, value) <-> x[userid] = value
        '''
        self.userdict[int(userid)] = value
    def __getitem__(self, userid):
        '''
        a = x.__getitem__(userid) <-> a = x[userid]
        '''
        if not self.userdict.has_key(int(userid)):
            nuser = playerlib.getPlayer(userid)
            if int(nuser.attributes['isbot']) == 0:
                self[int(userid)] = Popup_user(int(userid))
            else:
                return None
        return self.userdict[int(userid)]
    def __delitem__(self, userid):
        '''
        x.__delitem__(userid) <-> del x[userid]
        '''
        if self.userdict.has_key(int(userid)):
            del self.userdict[int(userid)]
    def haspopup(self, user):
        '''
        Some player instructs that he has popups in queue or active.
        If this is the first report, register es_client_command event handler
        '''
        es.dbgmsg(1,"Popuplib: haspopup!")
        if len(self.activepopups) == 0:
            es.dbgmsg(1,"Popuplib: register es_client_command")
            es.addons.registerForEvent(selfmodule, 'es_client_command', es_client_command)
        self.activepopups.add(user)
    def hasnopopup(self, user):
        '''
        Some player instructs that he no longer has any popups in queue or active.
        If this is the last report, unregister es_client_command event handler
        '''
        es.dbgmsg(1,"Popuplib: hasnopopup!")
        if user in self.activepopups:
            self.activepopups.remove(user)
            if len(self.activepopups) == 0:
                gamethread.delayed(1, self.checkeventneed)
    def checkeventneed(self):
        '''
        Checks if the es_client_command is still needed
        '''
        if len(self.activepopups) == 0:
            es.dbgmsg(1,"Popuplib: unregister es_client_command")
            es.addons.unregisterForEvent(selfmodule, 'es_client_command')



gUsers = UserHolderClass()


test = info
#popup commands begin here
#usage from other Python scripts for example:
#  import es
#  import popuplib
#  a = popuplib.create("insertnamehere")
#  a.addline("Line 1")
#  a.send(es.getuserid("Jonez"))
#  a.delete
def create(pPopupid):
    '''
    create a new normal popup
    '''
    if (pPopupid in gPopups):
        gPopups[pPopupid].__init__(pPopupid)
    else:
        gPopups[pPopupid] = Popup_popup(pPopupid)
    return gPopups[pPopupid]

def delete(pPopupid):
    '''
    delete a popup
    '''
    if (pPopupid in gPopups):
        del gPopups[pPopupid]
    else:
        raise ValueError("Popuplib: Cannot delete popup %s, it does not exist"%pPopupid)

def active(userid):
    '''
    check which popup is active and how many are in queue
    '''
    pl = {'name': '0', 'count': 0, 'object': None}
    user = gUsers[int(userid)]
    if not user:
        return None
    if user.active:
        pl['name']=user.active.name
        pl['count']=len(user.queue)+1
        pl['object']=user.active
    return pl

def exists(pPopupid):
    '''
    does named popup exist
    '''
    return (pPopupid in gPopups)

def find(pPopupid):
    '''
    return class instance of named popup
    '''
    if (pPopupid in gPopups):
        return gPopups[pPopupid]
    return None

def construct(pPopupid, pList, pFlags):
    '''
    create a new construct popup
    '''
    gPopups[pPopupid] = Popup_construct(pPopupid, pList, pFlags)
    return gPopups[pPopupid]

def easymenu(pPopupid, pVarn, pBlock):
    '''
    create a new easymenu popup
    '''
    gPopups[pPopupid] = Popup_easymenu(pPopupid, pVarn, pBlock)
    return gPopups[pPopupid]

def emulate(pPopupid, pItem, pUsers):
    '''
    emulate user doing a menu choice from any existing popup.
    use with caution...
    '''
    try:
        userid = int(pUsers)
        gPopups[pPopupid].handleChoice(int(pItem), userid)
    except TypeError:
        for userid in pUsers:
            gPopups[pPopupid].handleChoice(int(pItem), userid)
    
def quicksend(pViewtime, pUserid, pText, pBlock=""):
    '''
    like es.menu, but uses popup queue
    '''
    qs = Popup_popup("quicksend")
    qs.timeoutview = pViewtime
    qs.cache = pText
    qs.menuselect = pBlock
    qs.send(pUserid)

def send(pPopupid, pUserid, prio=False):
    '''
    send a named popup to user/users
    '''
    if pPopupid in gPopups:
        gPopups[pPopupid].send(pUserid,prio)
    else:
        raise ValueError("Popuplib: Cannot send popup %s, it does not exist"%pPopupid)

def unsend(pIndex, pUserid):
    '''
    INTERNAL (mostly)
    remove a queued popup by its queue index
    '''
    gUsers[pUserid].unsendSend(pIndex)
    
def unsendid(pIndex, pUserid):
    '''
    INTERNAL (mostly)
    remove a visible popup by its queue index
    '''
    gUsers[pUserid].unsendView(pIndex)
    
def unsendname(pName, pUsers):
    '''
    remove queued popups by the popup name
    '''
    try:
        userid = int(pUsers)
        gUsers[userid].unsendName(pName)
    except TypeError:
        for userid in pUsers:
            user = gUsers[int(userid)]
            if user: user.unsendName(pName)
    
def close(pName, pUsers):
    '''
    remove queued popups by the popup name but do not trigger menuselect 10
    '''
    try:
        userid = int(pUsers)
        gUsers[userid].unsendName(pName, True)
    except TypeError:
        for userid in pUsers:
            user = gUsers[int(userid)]
            if user: user.unsendName(pName, True)
    
def isqueued(pName, pUserid):
    '''
    returns the queue position of the named popup
    '''
    user = gUsers[pUserid]
    if user and pName in gPopups:
        return user.isqueued(gPopups[pName])

def langstring(prefix, langobject, suffix=''):
    '''
    constructs a language dict by adding prefix and suffix for each element
    '''
    newc = dict()
    for l in langobject:
        newc[l] = "%s%s%s"%(prefix,langobject[l],suffix)
    return newc

def _pnformat(s, n):
    '''
    INTERNAL function for formatting title using page numbering
    '''
    if n>0:
        l = len(s)
        a = max((1,n-l))
        x = "%s%s(%s)"%(s," "*a,"%p/%t")
        return x
    return s

######################
#EventScripts events #
######################

def es_map_start(event_var):
    es.loadevents("addons/eventscripts/popup/popup.res")
    gUsers.__init__()

def es_client_command(event_var):
    es.dbgmsg(1,"Popuplib: es_client_command")
    command = event_var['command']
    if command == "menuselect":
        try:
            choice = int(event_var['commandstring'])
            userid = int(event_var['userid'])
            user = gUsers[userid]
            if user.closemode:
                user.closemode = False
            else:
                active = user.active
                if active:
                    active.handleChoice(choice, userid)
            gUsers[userid].handleQueue()
        except ValueError:
            # It was not a popup menuselect
            pass


# Register for es_map_start to be able to use custom events
es.addons.registerForEvent(selfmodule, 'es_map_start', es_map_start)
# Declare popup events
es.loadevents("declare", "addons/eventscripts/popup/popup.res")

es_map_start(None)

########################
# GameThread functions #
########################

def gt_priofinish(user):
    user.prioritized = False
